﻿/** 
 * ---------------------------------------------------------------------------
 * BasePigmentTicket
 * ベースピグメントチケットクラス
 * ---------------------------------------------------------------------------
 * Rev.1  2017/06/14
 *   変更履歴
 *     ・クリーンアップ染料対応
 */
using System;
using Server.Targeting;

namespace Server.Items
{

    public abstract class BasePigmentTicket : Item
    {

        /** 分類コード */
        protected int m_category = 0;
        /** 染料名 */
        protected String m_PigmentName = "";
        /** 色 */
        protected int m_ColorCode = 0;
        /** 使用回数 */
        protected int m_Count = 0;

        public int PigCategory { get { return m_category; } set { m_category = value; InvalidateProperties(); } }
        public String PigName { get { return m_PigmentName; } set { m_PigmentName = value; InvalidateProperties(); } }
        public int PigColorCode { get { return m_ColorCode; } set { m_ColorCode = value; InvalidateProperties(); } }
        public int PigCount { get { return m_Count; } set { m_Count = value; InvalidateProperties(); } }

        [Constructable]
        public BasePigmentTicket()
                : base(0x2BE2)
        {
            this.Weight = 1.0;
        }
        public BasePigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void GetProperties(ObjectPropertyList list)
        {
            base.GetProperties(list);
            if (this.m_PigmentName != "")
            {
                list.Add(this.m_PigmentName);
                list.Add(1116466, this.m_Count.ToString());
            }
        }
        public override void OnDoubleClick(Mobile from)
        {
            if (IsChildOf(from.Backpack))
            {
                from.SendMessage("染色するアイテムを選択してください。");
                from.Target = new InternalTarget(this);
            }
        }

        private class InternalTarget : Target
        {
            private BasePigmentTicket m_BasePigmentTicket;

            public InternalTarget(BasePigmentTicket bpt)
                : base(-1, false, TargetFlags.None)
            {
                m_BasePigmentTicket = bpt;
            }
            protected override void OnTarget(Mobile from, object targeted)
            {
                if (from == null || targeted == null || !(targeted is Item)) return;
                Item useItem = (Item)targeted;
                if (LapisItemUtility.isDyable(m_BasePigmentTicket, useItem, from))
                {
                    from.PlaySound(0x23E);
                    useItem.Hue = m_BasePigmentTicket.Hue;
                    // Rev.1(S) ひそかになおす
                    //if (m_BasePigmentTicket.PigCount == 1)
                    if (m_BasePigmentTicket.PigCount <= 1)
                    {
                        m_BasePigmentTicket.Delete();
                    }
                    else
                    {
                        m_BasePigmentTicket.PigCount--;
                    }
                }
            }
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0); // version
            writer.Write(m_category);
            writer.Write(m_PigmentName);
            writer.Write(m_ColorCode);
            writer.Write(m_Count);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
            m_category = reader.ReadInt();
            m_PigmentName = reader.ReadString();
            m_ColorCode = reader.ReadInt();
            m_Count = reader.ReadInt();
            InvalidateProperties();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * BlankPigmentTicket
     * ブランクピグメントチケットクラス
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class BlankPigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public BlankPigmentTicket()
            : base()
        {
            this.Name = "a blank pigment ticket";
        }
        public BlankPigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void OnDoubleClick(Mobile from)
        {
            if (IsChildOf(from.Backpack))
            {
                from.SendMessage("染料または染料の元となる資材を選択してください。");
                from.Target = new InternalTarget(this);
            }
        }
        /**
         * ピグメントチケットを生成します
         */
        public void makePigmentTicket(int p_category, String p_Name, int p_Hue, int p_Count, Mobile player)
        {
            BasePigmentTicket bpt = null;
            int wk_cat = 1;
            if (p_category == wk_cat) bpt = new ClothPigmentTicket();   // 布                カテゴリ:1
            else if (p_category == ++wk_cat) bpt = new IngotPigmentTicket();   // 鉱石              カテゴリ:2
            else if (p_category == ++wk_cat) bpt = new LeatherPigmentTicket(); // 皮                カテゴリ:3
            else if (p_category == ++wk_cat) bpt = new BoardPigmentTicket();   // 木材              カテゴリ:4
            else if (p_category == ++wk_cat) bpt = new NaturalPigmentTicket(); // 自然染料          カテゴリ:5
            else if (p_category == ++wk_cat) bpt = new TAF1PigmentTicket();    // トクノ染料(TAF1)  カテゴリ:6
            else if (p_category == ++wk_cat) bpt = new TAF2PigmentTicket();    // トクノ染料(TAF2)  カテゴリ:7
            else if (p_category == ++wk_cat) bpt = new TMAF2PigmentTicket();   // トクノ染料(TMAF2) カテゴリ:8
            else if (p_category == ++wk_cat) bpt = new TMAF3PigmentTicket();   // トクノ染料(TMAF3) カテゴリ:9
            else if (p_category == ++wk_cat) bpt = new SpecialPigmentTicket(); // 特殊カラー        カテゴリ:10
            else if (p_category == ++wk_cat) bpt = new CleanUpPigmentTicket(); // クリーンアップ染料カテゴリ:11 Rev.1

            if (bpt == null)
            {
                return;
            }
            bpt.PigCategory = p_category;
            bpt.Hue = p_Hue;
            bpt.PigName = p_Name;
            bpt.PigCount = p_Count;
            player.Backpack.AddItem(bpt);
            this.Delete();
        }
        private class InternalTarget : Target
        {
            private BlankPigmentTicket m_BlankPigmentTicket;

            public InternalTarget(BlankPigmentTicket bpt)
                : base(-1, false, TargetFlags.None)
            {
                m_BlankPigmentTicket = bpt;
            }

            protected override void OnTarget(Mobile from, object targeted)
            {
                int p_category = 0;
                String p_Name = null;
                int p_Hue = 0;
                int p_Count = 0;
                // 重量判定
                if ((from.MaxWeight - from.TotalWeight) < 50)
                {
                    from.SendMessage("バックパックの重量に余裕がありません。");
                    return;
                }
                // 所持数判定
                if (from.Backpack.TotalItems > 124)
                {
                    from.SendMessage("バックパックの空きに余裕がありません。");
                    return;
                }
                // ターゲットされたのが自然染料である
                if (targeted is NaturalDye)
                {
                    NaturalDye nd = (NaturalDye)targeted;
                    for (int i = 0; i < LapisItemUtility.CstNaturalHue.Length; i++)
                    {
                        if (nd.Hue == LapisItemUtility.CstNaturalHue[i])
                        {
                            p_category = 5;
                            p_Hue = LapisItemUtility.CstNaturalHue[i];
                            p_Name = LapisItemUtility.CstNaturalName[i];
                            break;
                        }
                    }
                    p_Count = nd.UsesRemaining;
                    m_BlankPigmentTicket.makePigmentTicket(p_category, p_Name, p_Hue, p_Count, from);
                    nd.Delete();
                }
                // ターゲットされたのがトクノ染料である
                else if (targeted is BasePigmentsOfTokuno)
                {
                    BasePigmentsOfTokuno pot = (BasePigmentsOfTokuno)targeted;
                    // ターゲットされたのが漂白剤である
                    if (pot.Hue == 0x0000)
                    {
                        p_category = 10;
                        p_Name = LapisItemUtility.CstSpecialName[0];
                        p_Hue = LapisItemUtility.CstSpecialHue[0];
                        p_Count = pot.UsesRemaining;
                    }
                    // ターゲットされたのがトクノ染め(TAF)である
                    else if (pot is PigmentsOfTokuno)
                    {

                        for (int i = 0; i < LapisItemUtility.CstToT1AFHue.Length; i++)
                        {
                            if (pot.Hue == LapisItemUtility.CstToT1AFHue[i])
                            {
                                p_category = 6;
                                p_Hue = LapisItemUtility.CstToT1AFHue[i];
                                p_Name = LapisItemUtility.CstToT1AFName[i];
                                break;
                            }
                        }

                        // TAF1になかったらTAF2を検索
                        if (p_Hue == 0)
                        {
                            for (int i = 0; i < LapisItemUtility.CstToT2AFHue.Length; i++)
                            {
                                if (pot.Hue == LapisItemUtility.CstToT2AFHue[i])
                                {
                                    p_category = 7;
                                    p_Hue = LapisItemUtility.CstToT2AFHue[i];
                                    p_Name = LapisItemUtility.CstToT2AFName[i];
                                    break;
                                }
                            }
                        }
                        p_Count = pot.UsesRemaining;
                    }
                    // ターゲットされたのがToT2のTMAFトクノ染めである
                    else if (pot is MetalPigmentsOfTokuno)
                    {
                        for (int i = 0; i < LapisItemUtility.CstToT2TMAFHue.Length; i++)
                        {
                            if (pot.Hue == LapisItemUtility.CstToT2TMAFHue[i])
                            {
                                p_category = 8;
                                p_Hue = LapisItemUtility.CstToT2TMAFHue[i];
                                p_Name = LapisItemUtility.CstToT2TMAFName[i];
                                break;
                            }
                        }
                    }
                    // ターゲットされたのがToT3のTMAFトクノ染めである
                    else if (pot is LesserPigmentsOfTokuno)
                    {
                        for (int i = 0; i < LapisItemUtility.CstToT3TMAFHue.Length; i++)
                        {
                            if (pot.Hue == LapisItemUtility.CstToT3TMAFHue[i])
                            {
                                p_category = 9;
                                p_Hue = LapisItemUtility.CstToT3TMAFHue[i];
                                p_Name = LapisItemUtility.CstToT3TMAFName[i];
                                break;
                            }
                        }
                        p_Count = pot.UsesRemaining;
                    }
                    m_BlankPigmentTicket.makePigmentTicket(p_category, p_Name, p_Hue, p_Count, from);
                    pot.Delete();
                }
                // Rev.1(S)
                // ターゲットされたのがクリーンアップ染料である
                else if (targeted is CompassionPigment)
                {
                    CompassionPigment cp = (CompassionPigment)targeted;
                    for (int i = 0; i < LapisItemUtility.CstCleanupHue.Length; i++)
                    {
                        if (cp.Hue == LapisItemUtility.CstCleanupHue[i])
                        {
                            p_category = 11;
                            p_Hue = LapisItemUtility.CstCleanupHue[i];
                            p_Name = LapisItemUtility.CstCleanupName[i];
                            break;
                        }
                    }
                    p_Count = cp.UsesRemaining;
                    m_BlankPigmentTicket.makePigmentTicket(p_category, p_Name, p_Hue, p_Count, from);
                    cp.Delete();
                }
                // Rev.1(E)
                // 資材系ターゲットチェック
                else if (isResource(targeted))
                {
                    // アイテム鑑定スキル値を取得
                    double skillVal = from.Skills[SkillName.ItemID].Value;
                    // アイテム鑑定スキル値が80以上である
                    if (skillVal >= 80.0)
                    {
                        // 資材の数をチェック
                        Item item = (Item)targeted;
                        if (item.Amount < 100)
                        {
                            from.SendMessage("Pigment Ticketの作成には資材が100個必要です。");
                            return;
                        }
                        // ターゲットされたのが布である
                        if (item is UncutCloth)
                        {
                            // 布色チェック
                            for (int i = 0; i < LapisItemUtility.CstUncutClothHue.Length; i++)
                            {
                                if (item.Hue == LapisItemUtility.CstUncutClothHue[i])
                                {
                                    p_category = 1;
                                    p_Name = LapisItemUtility.CstUncutClothName[i];
                                    p_Hue = LapisItemUtility.CstUncutClothHue[i];
                                    break;
                                }
                            }
                            // 報酬布以外の布が選択されたら処理を終える
                            if (p_Name == null)
                            {
                                from.SendMessage("選択した布の色はPigment Ticketにすることができません。");
                                return;
                            }
                        }
                        // ターゲットされたのがインゴットであり、鉄ではない
                        else if (item is BaseIngot && !(item is IronIngot))
                        {
                            p_category = 2;
                            if (item is DullCopperIngot) p_Name = "DullCopper";
                            else if (item is ShadowIronIngot) p_Name = "ShadowIron";
                            else if (item is CopperIngot) p_Name = "Copper";
                            else if (item is BronzeIngot) p_Name = "Bronze";
                            else if (item is GoldIngot) p_Name = "Gold";
                            else if (item is AgapiteIngot) p_Name = "Agapite";
                            else if (item is VeriteIngot) p_Name = "Verite";
                            else if (item is ValoriteIngot) p_Name = "Valorite";
                            p_Hue = CraftResources.GetHue(((BaseIngot)item).Resource);
                        }
                        // ターゲットされたのが皮であり、通常の皮ではない
                        else if (item is BaseLeather && !(item is Leather))
                        {
                            p_category = 3;
                            if (item is SpinedLeather) p_Name = "SpinedLeather";
                            else if (item is HornedLeather) p_Name = "HornedLeather";
                            else if (item is BarbedLeather) p_Name = "BarbedLeather";
                            p_Hue = CraftResources.GetHue(((BaseLeather)item).Resource);
                        }
                        // ターゲットされたのが木材であり、通常の木材ではない
                        else if (item is BaseWoodBoard && !(item is Board))
                        {
                            p_category = 4;
                            if (item is OakBoard) p_Name = "Oak";
                            else if (item is AshBoard) p_Name = "Ash";
                            else if (item is YewBoard) p_Name = "Yew";
                            else if (item is BloodwoodBoard) p_Name = "Bloodwood";
                            else if (item is HeartwoodBoard) p_Name = "Heartwood";
                            else if (item is FrostwoodBoard) p_Name = "Frostwood";
                            p_Hue = CraftResources.GetHue(((BaseWoodBoard)item).Resource);

                        }
                        // アイテム鑑定スキル値によってチャージ数を変更
                        // スキル値がGM以上である
                        if (skillVal >= 100.0)
                        {
                            p_Count = 3;
                        }
                        // スキル値が90～99.9である
                        else if (skillVal >= 90.0)
                        {
                            p_Count = 2;
                        }
                        // スキル値が80～89.9である
                        else
                        {
                            p_Count = 1;
                        }

                        // アイテム消費
                        if (item.Amount == 100)
                        {
                            item.Delete();
                        }
                        else
                        {
                            item.Amount = item.Amount - 100;
                        }
                    }
                    else
                    {
                        from.SendMessage("資材を使用したPigment Ticketの作成はアイテム鑑定スキル80.0以上が必要です。");
                        return;
                    }
                    m_BlankPigmentTicket.makePigmentTicket(p_category, p_Name, p_Hue, p_Count, from);
                }
                else
                {
                    from.SendMessage("そういうイジワルはしちゃあかんです！！");
                }

            }

            /**
             * ターゲットされたのが特定の資材か判別します。
             */
            private bool isResource(object targeted)
            {
                if (
                    targeted is UncutCloth ||
                    (targeted is BaseWoodBoard && !(targeted is Board)) ||
                    (targeted is BaseIngot && !(targeted is IronIngot)) ||
                    (targeted is BaseLeather && !(targeted is Leather))

                    )
                {
                    return true;
                }
                return false;
            }
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * ClothPigmentTicket
     * 布ピグメントチケットクラス
     * カテゴリNoは1
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class ClothPigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public ClothPigmentTicket()
            : base()
        {
            this.Name = "a cloth pigment ticket";
        }
        public ClothPigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * IngotPigmentTicket
     * インゴットピグメントチケットクラス
     * カテゴリNoは2
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class IngotPigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public IngotPigmentTicket()
            : base()
        {
            this.Name = "an ingot pigment ticket";
        }
        public IngotPigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * LeatherPigmentTicket
     * 皮ピグメントチケットクラス
     * カテゴリNoは3
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class LeatherPigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public LeatherPigmentTicket()
            : base()
        {
            this.Name = "a leather pigment ticket";
        }
        public LeatherPigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * BoardPigmentTicket
     * 木材ピグメントチケットクラス
     * カテゴリNoは4
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class BoardPigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public BoardPigmentTicket()
            : base()
        {
            this.Name = "a board pigment ticket";
        }
        public BoardPigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * NaturalPigmentTicket
     * 自然染料ピグメントチケットクラス
     * カテゴリNoは5
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class NaturalPigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public NaturalPigmentTicket()
            : base()
        {
            this.Name = "a natural pigment ticket";
        }
        public NaturalPigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * TAF1PigmentTicket
     * トクノ染料（TAF）ピグメントチケットクラス
     * カテゴリNoは6
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class TAF1PigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public TAF1PigmentTicket()
            : base()
        {
            this.Name = "a TAF1 pigment ticket";
        }
        public TAF1PigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * TAF2PigmentTicket
     * トクノ染料（TAF）ピグメントチケットクラス
     * カテゴリNoは7
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class TAF2PigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public TAF2PigmentTicket()
            : base()
        {
            this.Name = "a TAF2 pigment ticket";
        }
        public TAF2PigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * TMAF2PigmentTicket
     * トクノ染料（TMAF TOT2）ピグメントチケットクラス
     * カテゴリNoは8
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class TMAF2PigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public TMAF2PigmentTicket()
            : base()
        {
            this.Name = "a TMAF2 pigment ticket";
        }
        public TMAF2PigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * TMAF3PigmentTicket
     * トクノ染料（TMAF TOT3）ピグメントチケットクラス
     * カテゴリNoは9
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class TMAF3PigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public TMAF3PigmentTicket()
            : base()
        {
            this.Name = "a TMAF3 pigment ticket";
        }
        public TMAF3PigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
    /** 
     * ---------------------------------------------------------------------------
     * SpecialPigmentTicket
     * 特殊ピグメントチケットクラス
     * カテゴリNoは10
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class SpecialPigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public SpecialPigmentTicket()
            : base()
        {
            this.Name = "a special pigment ticket";
        }
        public SpecialPigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }

    /** 
     * ---------------------------------------------------------------------------
     * CleanUpPigmentTicket
     * クリーンアップ染料ピグメントチケットクラス
     * カテゴリNoは11
     * ---------------------------------------------------------------------------
     * Rev.1  ----/--/--
     *   変更履歴
     */
    public class CleanUpPigmentTicket : BasePigmentTicket
    {
        [Constructable]
        public CleanUpPigmentTicket()
            : base()
        {
            this.Name = "a cleanup pigment ticket";
        }
        public CleanUpPigmentTicket(Serial serial)
            : base(serial)
        {
        }
        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);
            writer.Write((int)0);
        }
        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
        }
    }
}
